export declare class BaseApi {
    #private;
    /** Die URL des Servers. Alle Pfadangaben sind relativ zu dieser URL. */
    protected url: string;
    /** Der Anmeldename beim Server */
    protected username: string;
    /**
     * Erstellt eine neue API mit der übergebenen Konfiguration.
     *
     * @param {string} url - die URL des Servers: Alle Pfadangaben sind relativ zu dieser URL
     * @param {string} username - der Benutzername für den API-Zugriff
     * @param {string} password - das Kennwort des Benutzers für den API-Zugriff
     */
    protected constructor(url: string, username: string, password: string);
    getPDF(path: string): Promise<Blob>;
    getText(path: string): Promise<string>;
    getJSON(path: string): Promise<string>;
    postText(path: string, body: string | null): Promise<string>;
    postJSON(path: string, body: string | null): Promise<string>;
    patchText(path: string, body: string): Promise<void>;
    patchJSON(path: string, body: string): Promise<void>;
    putText(path: string, body: string): Promise<void>;
    putJSON(path: string, body: string): Promise<void>;
    deleteText(path: string): Promise<void>;
    deleteJSON(path: string): Promise<void>;
}
