import { JavaObject } from '../../java/lang/JavaObject';
import { Abschluss } from '../../core/types/Abschluss';
import { AbschlussErgebnisBerufsbildend } from '../../core/data/abschluss/AbschlussErgebnisBerufsbildend';
import { BKAnlageAFaecher } from '../../core/abschluss/bk/a/BKAnlageAFaecher';
export declare class AbschlussManagerBerufsbildend extends JavaObject {
    constructor();
    /**
     * Erzeugt ein Ergebnis der Abschlussberechnung unter Angabe, ob dieser erworben
     * wurde. Die Liste der Nachprüfungsfächer ist leer und ein Log ist nicht zugeordnet.
     *
     * @param hatBSA                      ist der Berufsschulabschluss erreicht
     * @param note                        Note des Abschlusses
     * @param hatBA                       ist der Berufsabschluss erreicht
     * @param abschlussAllgemeinbildend   der allgemeinbildende Abschluss
     *
     * @return das Ergebnis der Abschlussberechnung
     */
    static getErgebnis(hatBSA: boolean, note: number, hatBA: Boolean | null, abschlussAllgemeinbildend: Abschluss | null): AbschlussErgebnisBerufsbildend;
    /**
     * Berechnet den Notendurchschnitt aller Fächer
     *
     * @param abschluss_faecher   die Fächer für die Abschlussberechnung
     *
     * @return der Notendurchschnitt oder NaN im Fehlerfall
     */
    static getDurchschnitt(abschluss_faecher: BKAnlageAFaecher): number;
    /**
     * Berechnet die Anzahl der Defizite
     *
     * @param abschluss_faecher   die Fächer für die Abschlussberechnung
     *
     * @return die Anzahl der Defizite oder -1 im Fehlerfall
     */
    static getAnzahlDefizite(abschluss_faecher: BKAnlageAFaecher): number;
    /**
     * Berechnet die Anzahl der Note Ungenügend
     *
     * @param abschluss_faecher   die Fächer für die Abschlussberechnung
     *
     * @return die Anzahl der Note Ungenügend oder -1 im Fehlerfall
     */
    static getAnzahlUngenuegend(abschluss_faecher: BKAnlageAFaecher): number;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_abschluss_AbschlussManagerBerufsbildend(obj: unknown): AbschlussManagerBerufsbildend;
