import { GostBelegpruefungsArt } from '../../../../core/abschluss/gost/GostBelegpruefungsArt';
import { GostBelegpruefung } from '../../../../core/abschluss/gost/GostBelegpruefung';
import { AbiturdatenManager } from '../../../../core/abschluss/gost/AbiturdatenManager';
export declare class Fremdsprachen extends GostBelegpruefung {
    private fremdsprachen;
    private fremdsprachen_neu;
    private fremdsprachen_fortgefuehrt;
    private biliSachfaecher;
    private anzahl_schriftlich_durchgehend;
    /**
     * Erstellt eine neue Belegprüfung für dir Fremdsprachen.
     *
     * @param manager         der Daten-Manager für die Abiturdaten
     * @param pruefungs_art   die Art der durchzuführenden Prüfung (z.B. EF.1 oder GESAMT)
     */
    constructor(manager: AbiturdatenManager, pruefungs_art: GostBelegpruefungsArt);
    protected init(): void;
    protected pruefeEF1(): void;
    /**
     * EF.1: Prüft bei der Sprachenfolge, ob eine gemäß Sprachenfolge fortgeführte
     * Fremdsprache fehlerhafterweise als neu einsetzende Fremdsprache belegt wurde.
     */
    private pruefeEF1Sprachenfolge;
    /**
     * Prüft, ob eine gültige Fremdsprachenbelegung mit den EF.1-Wahlen möglich ist.
     */
    private pruefeEF1Fremdsprache1;
    /**
     * Prüft, ob eine zweite Fremdsprache in der Sek I vorhanden ist und prüft sonst auf eine neu
     * einsetzende Fremdsprache.
     */
    private pruefeEF1FremdsprachenfolgeZweiteFremdsprache;
    /**
     * Prüft, ob alle Fremdsprachen in der EF.1 schriftlich belegt wurden.
     */
    private pruefeEF1Schriftlichkeit;
    /**
     * Zähle alle Fremdsprachen, die durchgehend schriftlich belegt wurden.
     * Hierzu zählt auch die Unterrichtssprache eines bilingualen Sachfachs als zweite durchgehende
     * Fremdsprache, sofern dieses durchgehende und schriftlich belegt werden kann.
     */
    private pruefeEF1AnzahlDurchgehenedeSprachen;
    /**
     * Prüft, ob die Bedingungen für die Wahl eines bilingualen Sachfaches erfüllt sind, sofern eines
     * in der EF.1 belegt wurde.
     */
    private pruefeEF1BilingualeSachfaecher;
    /**
     * Prüfe, ob die Bedingungen für den bilingualen Bildungsgang erfüllt sind, sofern ein solcher vom Schüler gewählt wurde.
     */
    private pruefeEF1BilingualenBildungsgang;
    protected pruefeGesamt(): void;
    /**
     * Gesamt: Prüft bei der Sprachenfolge, ob eine laut Sprachenfolge fortgeführte
     * Fremdpsrache fehlerhafterweise als neu einsetzende Fremdsprache belegt wurde.
     */
    private pruefeGesamtSprachenfolge;
    /**
     * Prüft, ob eine gültige Fremdsprachenbelegung in Bezug auf eine durchgehende Belegung möglich ist.
     */
    private pruefeGesamtFremdsprache1;
    /**
     * Prüft, ob eine zweite Fremdsprache in der Sek I vorhanden ist und prüft sonst auf eine neu
     * einsetzende Fremdsprache.
     */
    private pruefeGesamtFremdsprachenfolgeZweiteFremdsprache;
    /**
     * Prüft, ob eine gültige Fremdsprachenbelegung in Bezug auf die Schriftlichkeit und LK-Wahl vohanden ist.
     */
    private pruefeGesamtSchriftlichkeit;
    /**
     * Zähle alle Fremdsprachen, die durchgehend schriftlich belegt wurden.
     * Hierzu zählt auch die Unterrichtssprache eines bilingualen Sachfachs als zweite durchgehende
     * Fremdsprache, sofern dieses durchgehend und schriftlich belegt wurde.
     */
    private pruefeGesamtAnzahlDurchgehenedeSprachen;
    /**
     * Prüft, ob die Bedingungen für die Wahl eines bilingualen Sachfaches erfüllt sind, sofern eines
     * belegt wurde.
     */
    private pruefeGesamtBilingualeSachfaecher;
    /**
     * Prüfe, ob die Bedingungen für ein bilinguales Abitur erfüllt sind, sofern ein solches vom
     * Schüler gewählt wurde.
     */
    private pruefeGesamtBilingualenBildungsgang;
    /**
     * Gibt die Anzahl der durchgehend schriftlich belegten bzw. belegbaren Fremdsprachen zurück.
     * Durchgehend schriftlich bedeutet, dass das Fach mind. von EF.1 bis Q2.1 schriftlich belegt wurde.
     * Hierfür kommen Fremdsprachen und ggf. ein bilinguales Sachfach in Frage, dessen Unterrichtssprache
     * nicht durchgehend schriftlich belegt.
     *
     * @return die Anzahl der durchgehend schriftlich belegten bzw. belegbaren Fremdsprachen zurück.
     */
    getAnzahlDurchgehendSchritflichBelegt(): number;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_abschluss_gost_belegpruefung_Fremdsprachen(obj: unknown): Fremdsprachen;
