import { GostBelegpruefungsArt } from '../../../../core/abschluss/gost/GostBelegpruefungsArt';
import { GostBelegpruefung } from '../../../../core/abschluss/gost/GostBelegpruefung';
import { AbiturdatenManager } from '../../../../core/abschluss/gost/AbiturdatenManager';
export declare class GesellschaftswissenschaftenUndReligion extends GostBelegpruefung {
    private gesellschaftswissenschaften;
    private geschichte;
    private sozialwissenschaften;
    private philosophie;
    private sonstige_gesellschaftswissenschaften;
    private religion;
    private zusatzkursFachbelegungen;
    /**
     * Erstellt eine neue Belegprüfung für den Bereich der Gesellschaftswissenschaften und Religion.
     *
     * @param manager         der Daten-Manager für die Abiturdaten
     * @param pruefungs_art   die Art der durchzuführenden Prüfung (z.B. EF.1 oder GESAMT)
     */
    constructor(manager: AbiturdatenManager, pruefungs_art: GostBelegpruefungsArt);
    protected init(): void;
    protected pruefeEF1(): void;
    /**
     * EF1-Prüfung Punkte 8-10:
     * Prüfe, ob eine Gesellschaftswissenschaft in EF.1 schriftlich belegt wurde und durchgängig belegbar ist
     *    und ob Geschichte belegt wurde
     *    und ob Sozialwissenschaften belegt wurde
     */
    private pruefeGesellschaftswissenschaftenEF1;
    /**
     * EF1-Prüfung Punkt 11:
     * Prüfe, ob Religion in EF.1 belegt wurde oder ob Philosophie und eine weitere durchgehend belegbare Gesellschaftswissenschaft belegt wurde.
     * Falls Philosophie als Ersatz für Religion gewählt wurde, zählt es nicht als durchgehend belegte Gesellschaftswissenschaft.
     */
    private pruefeReligionEF1;
    protected pruefeGesamt(): void;
    /**
     * Gesamtprüfung Punkt 39:
     * Prüfe, on in EF.1 und EF.2 jeweils ein Fach der Gesellschaftswissenschaften schriftlich belegt wurde
     */
    private pruefeSchriftlichkeitEF;
    /**
     * Gesamtprüfung Punkt 38:
     * Prüfe, ob ein Fach der Gesellschaftswissenschaften von EF.1 bis Q2.2 durchgängig belegt wurde
     * - Zusatzkurse zählen hier nicht als Belegung
     */
    private pruefeDurchgaengigeBelegung;
    /**
     * Gesamtprüfung Punkt 40:
     * Prüfe, ob ein Fach der Gesellschaftswissenschaften oder Religionslehre von EF.1 bis Q2.2 belegt
     * und von Q1.1 bis Q2.1 schriftlich belegt wurde, damit es als potentielles Abiturfach zur Verfügung steht.
     * - Zusatzkurse zählen hier nicht als Belegung
     */
    private pruefeDurchgaengigeBelegungUndSchriftlich;
    /**
     * Gesamtprüfung Punkte 27, 29 und 30:
     * Prüft, ob der Zusatzkurs genau zwei mal belegt wurde
     *    und ob ein Zusatzkurs belegt wurde, obwohl im Halbjahr zuvor ein Geschichtskurs belegt wurde.
     *
     * @param fachbelegungen   die Fachbelegung für Geschichte oder Sozialwissehschaften
     */
    private pruefeZusatzkurs;
    /**
     * Gesamtprüfung Punkt 41:
     * Prüft, ob Geschichte korrekt belegt wurde (mind. von EF.1 bis Q1.2 oder als Zusatzkurs)
     */
    private pruefeBelegungGeschichte;
    /**
     * Gesamtprüfung Punkt 42:
     * Prüft, ob Sozialwissenschaften korrekt belegt wurde (mind. von EF.1 bis Q1.2 oder als Zusatzkurs)
     */
    private pruefeBelegungSozialwissenschaften;
    /**
     * Gesamtprüfung Punkte 43:
     * Prüft die Belegung von Religion und Philosophie in der EF. Wird Philosophie als Ersatz belegt, so wird auch geprüft,
     * ob eine weitere Gesellschaftswissenschaft belegt wurde.
     */
    private pruefeReligionEF;
    /**
     * Gesamtprüfung Punkt 44:
     * Prüft die Belegung von Religion und Philosophie. Wird Philosophie als Ersatz belegt, so wird auch geprüft, ob eine weitere
     * Gesellschaftswissenschaft belegt wurde.
     */
    private pruefeReligionQ1;
    /**
     * Gesamtprüfung Punkt 55:
     * Prüft, ob Fehler bei der Kontinuität bei Philosophie und Religion nur durch die Ersatzfachregelung bei Religion
     * zustandekommen und damit zulässig sind.
     */
    private pruefeReligionKontinuitaet;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_abschluss_gost_belegpruefung_GesellschaftswissenschaftenUndReligion(obj: unknown): GesellschaftswissenschaftenUndReligion;
