import { JavaObject } from '../../java/lang/JavaObject';
import { KursblockungDynStatistik } from '../../core/kursblockung/KursblockungDynStatistik';
import { KursblockungDynKurs } from '../../core/kursblockung/KursblockungDynKurs';
import { Logger } from '../../logger/Logger';
export declare class KursblockungDynSchiene extends JavaObject {
    private readonly nr;
    private readonly logger;
    private readonly kursMap;
    private readonly statistik;
    /**
     * Im Konstruktor werden die Referenzen übernommen und das HashMap erzeugt.
     *
     * @param pLogger    Logger für Benutzerhinweise, Warnungen und Fehler.
     * @param pNr        Die Nummer der Schiene.
     * @param pStatistik Das Statistik-Objekt wird über die aktuellen Kurs-Paarungen
     *                   informiert.
     */
    constructor(pLogger: Logger, pNr: number, pStatistik: KursblockungDynStatistik);
    toString(): String;
    /**
     * Fügt der Schiene einen Kurs hinzu. Das Statistik-Objekt wird über neue
     * Kurs-Paarungen informiert.
     *
     * @param kurs1 Der Kurs, welcher der Schiene hinzugefügt werden soll.
     */
    aktionKursHinzufuegen(kurs1: KursblockungDynKurs): void;
    /**
     * Entfernt aus der Schiene einen Kurs. Das Statistik-Objekt wird über zu
     * entfernende Kurs-Paarungen informiert.
     *
     * @param kurs1 Der Kurs, welcher aus der Schiene entfernt werden soll.
     */
    aktionKursEntfernen(kurs1: KursblockungDynKurs): void;
    /**
     * Liefert die aktuelle Nummer der Schiene (0-indiziert).
     *
     * @return Die aktuelle Nummer der Schiene (0-indiziert).
     */
    gibNr(): number;
    /**
     * Liefert die aktuelle Anzahl an Kursen in dieser Schiene.
     *
     * @return Die aktuelle Anzahl an Kursen in dieser Schiene.
     */
    gibKursAnzahl(): number;
    /**
     * Debug-Ausgabe. Nur für Testzwecke.
     * @param nurMultikurse Fallse TRUE, werden nur Multikurse angezeigt.
     */
    debug(nurMultikurse: boolean): void;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_kursblockung_KursblockungDynSchiene(obj: unknown): KursblockungDynSchiene;
