import { JavaObject } from '../../java/lang/JavaObject';
export declare class Geschlecht extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly M: Geschlecht;
    static readonly W: Geschlecht;
    static readonly D: Geschlecht;
    static readonly X: Geschlecht;
    readonly id: number;
    readonly kuerzel: String;
    readonly text: String;
    readonly textLang: String;
    /**
     * Erzeugt ein neues Geschlecht für die Aufzählung der Geschlechter.
     *
     * @param id          die ID des Geschlechtes, welche im Rahmen der amtlichen Schulstatistik verwendet wird
     * @param kuerzel     das Geschlecht als einstelliges Kürzel
     * @param text        die textuelle Kurz-Bezeichnung des Geschlechtes
     * @param textLang    die ausführliche textuelle Bezeichnung des Geschlechtes
     */
    private constructor();
    /**
     *
     * Bestimmt das Geschlecht anhand der ID.
     *
     * @param value   die ID des Geschlechtes
     *
     * @return das Geschlecht oder null, falls die ID fehlerhaft ist
     */
    static fromValue(value: Number | null): Geschlecht | null;
    /**
     *
     * Bestimmt das Geschlecht anhand des übergebenen Strings.
     * Enthält der übegebene String einen ungültigen Wert,
     * so wird als Geschlecht "x", d.h. ohne Angabe im Geburtenregister
     * zurückgegebeben.
     *
     * @param text         die textuelle Beschreibung des Geschlechts
     *
     * @return das Geschlecht als Type
     */
    static fromStringValue(text: String | null): Geschlecht;
    /**
     * Gibt die Anrede für eine Person dieses Geschlechts in Abhängigkeit vom Alter zurück.
     *
     * @param alter   das Alter der Person
     *
     * @return die Anrede
     */
    getAnrede(alter: number): String | null;
    toString(): String;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: Geschlecht): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<Geschlecht>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): Geschlecht | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_Geschlecht(obj: unknown): Geschlecht;
