"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_Geschlecht = exports.Geschlecht = void 0;
const JavaObject_1 = require("../../java/lang/JavaObject");
class Geschlecht extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static M = new Geschlecht("M", 0, 3, "m", "männlich", "männlich");
    static W = new Geschlecht("W", 1, 4, "w", "weiblich", "weiblich");
    static D = new Geschlecht("D", 2, 5, "d", "divers", "divers");
    static X = new Geschlecht("X", 3, 6, "x", "ohne Angabe", "ohne Angabe im Geburtenregister");
    id;
    kuerzel;
    text;
    textLang;
    /**
     * Erzeugt ein neues Geschlecht für die Aufzählung der Geschlechter.
     *
     * @param id          die ID des Geschlechtes, welche im Rahmen der amtlichen Schulstatistik verwendet wird
     * @param kuerzel     das Geschlecht als einstelliges Kürzel
     * @param text        die textuelle Kurz-Bezeichnung des Geschlechtes
     * @param textLang    die ausführliche textuelle Bezeichnung des Geschlechtes
     */
    constructor(name, ordinal, id, kuerzel, text, textLang) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        Geschlecht.all_values_by_ordinal.push(this);
        Geschlecht.all_values_by_name.set(name, this);
        this.id = id;
        this.kuerzel = kuerzel;
        this.text = text;
        this.textLang = textLang;
    }
    /**
     *
     * Bestimmt das Geschlecht anhand der ID.
     *
     * @param value   die ID des Geschlechtes
     *
     * @return das Geschlecht oder null, falls die ID fehlerhaft ist
     */
    static fromValue(value) {
        if (value === null)
            return null;
        switch (value) {
            case 3:
                return Geschlecht.M;
            case 4:
                return Geschlecht.W;
            case 5:
                return Geschlecht.D;
            case 6:
                return Geschlecht.X;
        }
        return null;
    }
    /**
     *
     * Bestimmt das Geschlecht anhand des übergebenen Strings.
     * Enthält der übegebene String einen ungültigen Wert,
     * so wird als Geschlecht "x", d.h. ohne Angabe im Geburtenregister
     * zurückgegebeben.
     *
     * @param text         die textuelle Beschreibung des Geschlechts
     *
     * @return das Geschlecht als Type
     */
    static fromStringValue(text) {
        if ((text === null) || JavaObject_1.JavaObject.equalsTranspiler("", (text)))
            return Geschlecht.X;
        let upperValue = text.toUpperCase();
        switch (upperValue) {
            case "MÄNNLICH":
            case "MAENNLICH":
            case "M":
                return Geschlecht.M;
            case "WEIBLICH":
            case "W":
                return Geschlecht.W;
            case "DIVERS":
            case "D":
                return Geschlecht.D;
            case "-":
            case "X":
            case "OHNE ANGABE":
            case "OHNE_ANGABE":
            case "OHNE ANGABE IM GEBURTENREGISTER":
                return Geschlecht.X;
        }
        return Geschlecht.X;
    }
    /**
     * Gibt die Anrede für eine Person dieses Geschlechts in Abhängigkeit vom Alter zurück.
     *
     * @param alter   das Alter der Person
     *
     * @return die Anrede
     */
    getAnrede(alter) {
        switch (this.id) {
            case 3:
                return (alter < 18) ? "Lieber" : "Sehr geehrter Herr";
            case 4:
                return (alter < 18) ? "Liebe" : "Sehr geehrte Frau";
            case 5:
                return "Guten Tag";
            case 6:
                return "Guten Tag";
        }
        return "Guten Tag";
    }
    toString() {
        return this.kuerzel;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof Geschlecht))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.Geschlecht'].includes(name);
    }
}
exports.Geschlecht = Geschlecht;
function cast_de_nrw_schule_svws_core_types_Geschlecht(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_Geschlecht = cast_de_nrw_schule_svws_core_types_Geschlecht;
//# sourceMappingURL=Geschlecht.js.map