import { JavaObject } from '../../java/lang/JavaObject';
export declare class KursFortschreibungsart extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly KEINE: KursFortschreibungsart;
    static readonly NUR_DEFINITION_JAHRGANG_HOCHSCHREIBEN: KursFortschreibungsart;
    static readonly NUR_DEFINITION_JAHRGANG_BEIBEHALTEN: KursFortschreibungsart;
    static readonly KOMPLETT: KursFortschreibungsart;
    static VERSION: number;
    readonly id: number;
    readonly kuerzel: String;
    readonly beschreibung: String;
    readonly gueltigVon: Number | null;
    readonly gueltigBis: Number | null;
    /**
     * Erstellt eine neue Kurs-Fortschreibungsart in der Aufzählung.
     *
     * @param id             die ID der Fortchreibungsart
     * @param kuerzel        das eindeutige einstelleige Kürzel der Kurs-Fortschreibungsart
     * @param beschreibung   die Beschreibung der Kurs-Fortschreibungsart
     * @param gueltigVon     gibt an, in welchem Schuljahr die Fortschreibungsart einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis     gibt an, bis zu welchem Schuljahr die Fortschreibungsart gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    private constructor();
    /**
     * Gibt die Kurs-Fortschreibungsart anhand der ID zurück.
     * Eine ungültige ID wird als Fortschreibungsart KEINE interpretiert.
     *
     * @param id    die ID
     *
     * @return die Kurs-Fortschreibungsart
     */
    static fromID(id: Number | null): KursFortschreibungsart | null;
    /**
     * Gibt die Kurs-Fortschreibungsart anhand des Kürzels zurück.
     * Eine ungültiges Kürzel wird als Fortschreibungsart KEINE interpretiert.
     *
     * @param kuerzel    das Kürzel
     *
     * @return die Kurs-Fortschreibungsart
     */
    static fromKuerzel(kuerzel: String | null): KursFortschreibungsart | null;
    /**
     * Prüft, ob das übergebene Kürzel für eine gültige Kurs-Fortschreibungsart
     * steht oder nicht
     *
     * @param kuerzel   das zu prüfende Kürzel
     *
     * @return true, falls das kürzel gültig ist.
     */
    static isValidKuerzel(kuerzel: String | null): boolean;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: KursFortschreibungsart): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<KursFortschreibungsart>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): KursFortschreibungsart | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_KursFortschreibungsart(obj: unknown): KursFortschreibungsart;
