"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_KursFortschreibungsart = exports.KursFortschreibungsart = void 0;
const JavaObject_1 = require("../../java/lang/JavaObject");
class KursFortschreibungsart extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static KEINE = new KursFortschreibungsart("KEINE", 0, 0, "N", "Keine", null, null);
    static NUR_DEFINITION_JAHRGANG_HOCHSCHREIBEN = new KursFortschreibungsart("NUR_DEFINITION_JAHRGANG_HOCHSCHREIBEN", 1, 1, "D", "Nur Definition, Jahrgang hochschreiben", null, null);
    static NUR_DEFINITION_JAHRGANG_BEIBEHALTEN = new KursFortschreibungsart("NUR_DEFINITION_JAHRGANG_BEIBEHALTEN", 2, 2, "B", "Nur Definition, Jahrgang beibehalten", null, null);
    static KOMPLETT = new KursFortschreibungsart("KOMPLETT", 3, 3, "K", "Komplett", null, null);
    static VERSION = 1;
    id;
    kuerzel;
    beschreibung;
    gueltigVon;
    gueltigBis;
    /**
     * Erstellt eine neue Kurs-Fortschreibungsart in der Aufzählung.
     *
     * @param id             die ID der Fortchreibungsart
     * @param kuerzel        das eindeutige einstelleige Kürzel der Kurs-Fortschreibungsart
     * @param beschreibung   die Beschreibung der Kurs-Fortschreibungsart
     * @param gueltigVon     gibt an, in welchem Schuljahr die Fortschreibungsart einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis     gibt an, bis zu welchem Schuljahr die Fortschreibungsart gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    constructor(name, ordinal, id, kuerzel, beschreibung, gueltigVon, gueltigBis) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        KursFortschreibungsart.all_values_by_ordinal.push(this);
        KursFortschreibungsart.all_values_by_name.set(name, this);
        this.id = id;
        this.kuerzel = kuerzel;
        this.beschreibung = beschreibung;
        this.gueltigVon = gueltigVon;
        this.gueltigBis = gueltigBis;
    }
    /**
     * Gibt die Kurs-Fortschreibungsart anhand der ID zurück.
     * Eine ungültige ID wird als Fortschreibungsart KEINE interpretiert.
     *
     * @param id    die ID
     *
     * @return die Kurs-Fortschreibungsart
     */
    static fromID(id) {
        if (id === null)
            return KursFortschreibungsart.KEINE;
        switch (id) {
            case 0:
                return KursFortschreibungsart.KEINE;
            case 1:
                return KursFortschreibungsart.NUR_DEFINITION_JAHRGANG_HOCHSCHREIBEN;
            case 2:
                return KursFortschreibungsart.NUR_DEFINITION_JAHRGANG_BEIBEHALTEN;
            case 3:
                return KursFortschreibungsart.KOMPLETT;
        }
        return KursFortschreibungsart.KEINE;
    }
    /**
     * Gibt die Kurs-Fortschreibungsart anhand des Kürzels zurück.
     * Eine ungültiges Kürzel wird als Fortschreibungsart KEINE interpretiert.
     *
     * @param kuerzel    das Kürzel
     *
     * @return die Kurs-Fortschreibungsart
     */
    static fromKuerzel(kuerzel) {
        if (kuerzel === null)
            return KursFortschreibungsart.KEINE;
        switch (kuerzel) {
            case "N":
                return KursFortschreibungsart.KEINE;
            case "D":
                return KursFortschreibungsart.NUR_DEFINITION_JAHRGANG_HOCHSCHREIBEN;
            case "B":
                return KursFortschreibungsart.NUR_DEFINITION_JAHRGANG_BEIBEHALTEN;
            case "K":
                return KursFortschreibungsart.KOMPLETT;
        }
        return KursFortschreibungsart.KEINE;
    }
    /**
     * Prüft, ob das übergebene Kürzel für eine gültige Kurs-Fortschreibungsart
     * steht oder nicht
     *
     * @param kuerzel   das zu prüfende Kürzel
     *
     * @return true, falls das kürzel gültig ist.
     */
    static isValidKuerzel(kuerzel) {
        for (let art of KursFortschreibungsart.values())
            if (JavaObject_1.JavaObject.equalsTranspiler(art.kuerzel, (kuerzel)))
                return true;
        return false;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof KursFortschreibungsart))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.KursFortschreibungsart'].includes(name);
    }
}
exports.KursFortschreibungsart = KursFortschreibungsart;
function cast_de_nrw_schule_svws_core_types_KursFortschreibungsart(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_KursFortschreibungsart = cast_de_nrw_schule_svws_core_types_KursFortschreibungsart;
//# sourceMappingURL=KursFortschreibungsart.js.map