import { JavaObject } from '../../java/lang/JavaObject';
export declare class Note extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly KEINE: Note;
    static readonly UNGENUEGEND: Note;
    static readonly MANGELHAFT_MINUS: Note;
    static readonly MANGELHAFT: Note;
    static readonly MANGELHAFT_PLUS: Note;
    static readonly AUSREICHEND_MINUS: Note;
    static readonly AUSREICHEND: Note;
    static readonly AUSREICHEND_PLUS: Note;
    static readonly BEFRIEDIGEND_MINUS: Note;
    static readonly BEFRIEDIGEND: Note;
    static readonly BEFRIEDIGEND_PLUS: Note;
    static readonly GUT_MINUS: Note;
    static readonly GUT: Note;
    static readonly GUT_PLUS: Note;
    static readonly SEHR_GUT_MINUS: Note;
    static readonly SEHR_GUT: Note;
    static readonly SEHR_GUT_PLUS: Note;
    static readonly ATTEST: Note;
    static readonly E1_MIT_BESONDEREM_ERFOLG_TEILGENOMMEN: Note;
    static readonly E2_MIT_ERFOLG_TEILGENOMMEN: Note;
    static readonly E3_TEILGENOMMEN: Note;
    static readonly NICHT_TEILGENOMMEN: Note;
    static readonly NICHT_BEURTEILT: Note;
    static readonly NICHT_ERTEILT: Note;
    static readonly LEHRERMANGEL: Note;
    static readonly ABGEMELDET: Note;
    static VERSION: number;
    readonly id: number;
    readonly sortierung: number;
    readonly notenpunkte: Number | null;
    readonly kuerzel: String;
    readonly text: String;
    readonly textZeugnis: String;
    readonly gueltigVon: Number | null;
    readonly gueltigBis: Number | null;
    /**
     * Erzeugt ein neues Element der Aufzählung
     *
     * @param id              die eindeutige ID der Note
     * @param sortierung      eine ID, die der Sortierung der Noteneinträge in einer Anwendung vorgibt
     * @param notenpunkte     die Notenpunkte, die dieser Note zugeordnet sind
     * @param kuerzel         die Kurzschreibweise der Note als Zahl ggf. mit Tendenz (+/-)
     * @param text            die Note in ausführlicher Textform ggf. mit Tendenz (plus/minus)
     * @param textZeugnis     die Note in ausführlicher Textform, wie sie auf einem Zeugnis dargestellt wird
     * @param gueltigVon      gibt an, in welchem Schuljahr die Note einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis      gibt an, bis zu welchem Schuljahr die Note gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    private constructor();
    /**
     * Gibt zurück, ob es sich um eine echte Note oder nur um eine "Pseudonote" handelt
     *
     * @return true, wenn es sich bei der Note um eine echte Note handelt
     */
    istNote(): boolean;
    /**
     * Bestimmt die Note anhand des übergebenen Integer-Wert, welcher eine
     * Note ohne Tendenz darstellt.
     *
     * @param noteSekI    die Note ohne Tendenz
     *
     * @return die Note
     */
    static fromNoteSekI(noteSekI: Number | null): Note | null;
    /**
     * Gibt die Note anhand der angebenen Notenpunkte zurück.
     *
     * @param notenpunkte   die Notenpunkte anhand derer die Note ermittelt wird
     *
     * @return die Note aus dieser Aufzählung oder Note.KEINE im Fehlerfall
     */
    static fromNotenpunkte(notenpunkte: Number | null): Note;
    /**
     * Gibt die Note bzw. Pseudonote anhand des angebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel anhand derer die Note ermittelt wird
     *
     * @return die Note aus dieser Aufzählung oder Note.KEINE im Fehlerfall
     */
    static fromKuerzel(kuerzel: String | null): Note;
    /**
     * Gibt die Note anhand der angebenen Notenpunkte zurück, welche als String
     * übergeben werden.
     *
     * @param notenpunkte   die Notenpunkte anhand derer die Note ermittelt wird als String
     *
     * @return die Note aus dieser Aufzählung oder Note.KEINE im Fehlerfall
     */
    static fromNotenpunkteString(notenpunkte: String | null): Note;
    /**
     * Gibt an, ob es sich um eine Note mit Tendenz handelt oder nicht.
     *
     * @return true, falls die Note eine Tendenz hat
     */
    hatTendenz(): boolean;
    /**
     * Ermittelt die zu der Note gehörende Note ohne Tendenz (z.B. 3+ wird zu 3)
     *
     * @return die entsprechende Note ohne Tendenz
     */
    ohneTendenz(): Note;
    toString(): String;
    /**
     * Gibt die Note als Note der Sekundarstufe I ohne Tendenz als Zahl zurück.
     *
     * @return die Noten 1-6 oder im Fehlerfall null
     */
    getNoteSekI(): Number | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: Note): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<Note>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): Note | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_Note(obj: unknown): Note;
