import { JavaObject } from '../../java/lang/JavaObject';
export declare class PersonalTyp extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly LEHRKRAFT: PersonalTyp;
    static readonly SEKRETARIAT: PersonalTyp;
    static readonly PERSONAL: PersonalTyp;
    static readonly EXTERN: PersonalTyp;
    static readonly SONSTIGE: PersonalTyp;
    static VERSION: number;
    private static readonly _mapBezeichnungen;
    private static readonly _mapID;
    readonly id: number;
    readonly kuerzel: String;
    readonly bezeichnung: String;
    readonly gueltigVon: Number | null;
    readonly gueltigBis: Number | null;
    /**
     * Erzeugt einen neuen PersonalTyp für die Aufzählung.
     *
     * @param id           die ID des Personal-Typs
     * @param kuerzel      das Kürzel des Personal-Typs
     * @param bezeichnung  die Bezeichnung des Personal-Typs
     * @param gueltigVon   gibt an, in welchem Schuljahr der Personaltyp einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis   gibt an, bis zu welchem Schuljahr der Personaltyp gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    private constructor();
    /**
     * Gibt eine Map von den IDs der Personal-Typen auf die zugehörigen Personal-Typen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Personal-Typen auf die zugehörigen Personal-Typen
     */
    private static getMapID;
    /**
     * Gibt eine Map von den Bezeichnungen der Personal-Typen auf die zugehörigen Personal-Typen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Bezeichnungen der Personal-Typen auf die zugehörigen Personal-Typen
     */
    private static getMapBezeichnungen;
    /**
     *
     * Gibt den PersonalTyp anhand der Bezeichnung zurück.
     *
     * @param bezeichnung   die Bezeichnung des Personal-Typs
     *
     * @return der Personal-Typ oder null, falls die Bezeichnung ungültig ist
     *
     */
    static fromBezeichnung(bezeichnung: String | null): PersonalTyp | null;
    /**
     *
     * Gibt den PersonalTyp anhand der ID zurück.
     *
     * @param id   die ID des Personal-Typs
     *
     * @return der Personal-Typ oder null, falls die ID ungültig ist
     *
     */
    static fromID(id: Number | null): PersonalTyp | null;
    toString(): String;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: PersonalTyp): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<PersonalTyp>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): PersonalTyp | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_PersonalTyp(obj: unknown): PersonalTyp;
