import { JavaObject } from '../../java/lang/JavaObject';
export declare class SchuelerStatus extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly NEUAUFNAHME: SchuelerStatus;
    static readonly WARTELISTE: SchuelerStatus;
    static readonly AKTIV: SchuelerStatus;
    static readonly BEURLAUBT: SchuelerStatus;
    static readonly EXTERN: SchuelerStatus;
    static readonly ABSCHLUSS: SchuelerStatus;
    static readonly ABGANG: SchuelerStatus;
    static readonly EHEMALIGE: SchuelerStatus;
    static VERSION: number;
    private static readonly _mapID;
    private static readonly _mapBezeichnungen;
    readonly id: number;
    readonly bezeichnung: String;
    readonly gueltigVon: Number | null;
    readonly gueltigBis: Number | null;
    /**
     * Erzeugt einen neuen Schüler-Status in der Aufzählung.
     *
     * @param id            die ID des Schüler Status, welche auch in der SVWS-Datenbank genutzt wird
     * @param bezeichnung   die textuelle Bezeichnung des Schülerstatus
     * @param gueltigVon    gibt an, in welchem Schuljahr der Schülerstatus einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis    gibt an, bis zu welchem Schuljahr der Schülerstatus gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    private constructor();
    /**
     * Gibt eine Map von den IDs der Schüler-Status auf die zugehörigen Schüler-Status
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Schüler-Status auf die zugehörigen Schüler-Status
     */
    private static getMapID;
    /**
     * Gibt eine Map von den Bezeichnungen der Schüler-Status auf die zugehörigen Schüler-Status
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Bezeichnungen der Schüler-Status auf die zugehörigen Schüler-Status
     */
    private static getMapBezeichnungen;
    /**
     *
     * Gibt den Schülerstatus anhand der ID zurück.
     *
     * @param status	die id des Schülerstatus
     *
     * @return der Schülerstatus oder null, falls die ID ungültig ist
     *
     */
    static fromID(status: number): SchuelerStatus | null;
    /**
     *
     * Ermittelt den Schülerstatus anhand der Bezeichnung.
     *
     * @param value	  die Bezeichnung des Schülerstatus
     *
     * @return der Schülerstatus oder null, falls die Bezeichnung ungültig ist
     *
     */
    static fromBezeichnung(value: String | null): SchuelerStatus | null;
    /**
     * Prüft, ob die übergebene ID für einen gültigen Schülerstatus
     * steht oder nicht
     *
     * @param id   die zu prüfende ID
     *
     * @return true, falls die ID gültig ist.
     */
    static isValidID(id: Number | null): boolean;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: SchuelerStatus): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<SchuelerStatus>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): SchuelerStatus | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_SchuelerStatus(obj: unknown): SchuelerStatus;
