"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_SchuelerStatus = exports.SchuelerStatus = void 0;
const JavaObject_1 = require("../../java/lang/JavaObject");
const HashMap_1 = require("../../java/util/HashMap");
class SchuelerStatus extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static NEUAUFNAHME = new SchuelerStatus("NEUAUFNAHME", 0, 0, "Neuaufnahme", null, null);
    static WARTELISTE = new SchuelerStatus("WARTELISTE", 1, 1, "Warteliste", null, null);
    static AKTIV = new SchuelerStatus("AKTIV", 2, 2, "Aktiv", null, null);
    static BEURLAUBT = new SchuelerStatus("BEURLAUBT", 3, 3, "Beurlaubt", null, null);
    static EXTERN = new SchuelerStatus("EXTERN", 4, 6, "Extern", null, null);
    static ABSCHLUSS = new SchuelerStatus("ABSCHLUSS", 5, 8, "Abschluss", null, null);
    static ABGANG = new SchuelerStatus("ABGANG", 6, 9, "Abgang", null, null);
    static EHEMALIGE = new SchuelerStatus("EHEMALIGE", 7, 10, "Ehemalige", null, null);
    static VERSION = 1;
    static _mapID = new HashMap_1.HashMap();
    static _mapBezeichnungen = new HashMap_1.HashMap();
    id;
    bezeichnung;
    gueltigVon;
    gueltigBis;
    /**
     * Erzeugt einen neuen Schüler-Status in der Aufzählung.
     *
     * @param id            die ID des Schüler Status, welche auch in der SVWS-Datenbank genutzt wird
     * @param bezeichnung   die textuelle Bezeichnung des Schülerstatus
     * @param gueltigVon    gibt an, in welchem Schuljahr der Schülerstatus einführt wurde. Ist kein Schuljahr bekannt, so ist null gesetzt.
     * @param gueltigBis    gibt an, bis zu welchem Schuljahr der Schülerstatus gültig ist. Ist kein Schuljahr bekannt, so ist null gesetzt.
     */
    constructor(name, ordinal, id, bezeichnung, gueltigVon, gueltigBis) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        SchuelerStatus.all_values_by_ordinal.push(this);
        SchuelerStatus.all_values_by_name.set(name, this);
        this.id = id;
        this.bezeichnung = bezeichnung;
        this.gueltigVon = gueltigVon;
        this.gueltigBis = gueltigBis;
    }
    /**
     * Gibt eine Map von den IDs der Schüler-Status auf die zugehörigen Schüler-Status
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Schüler-Status auf die zugehörigen Schüler-Status
     */
    static getMapID() {
        if (SchuelerStatus._mapID.size() === 0)
            for (let p of SchuelerStatus.values())
                SchuelerStatus._mapID.put(p.id, p);
        return SchuelerStatus._mapID;
    }
    /**
     * Gibt eine Map von den Bezeichnungen der Schüler-Status auf die zugehörigen Schüler-Status
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Bezeichnungen der Schüler-Status auf die zugehörigen Schüler-Status
     */
    static getMapBezeichnungen() {
        if (SchuelerStatus._mapBezeichnungen.size() === 0)
            for (let p of SchuelerStatus.values())
                SchuelerStatus._mapBezeichnungen.put(p.bezeichnung.toUpperCase(), p);
        return SchuelerStatus._mapBezeichnungen;
    }
    /**
     *
     * Gibt den Schülerstatus anhand der ID zurück.
     *
     * @param status	die id des Schülerstatus
     *
     * @return der Schülerstatus oder null, falls die ID ungültig ist
     *
     */
    static fromID(status) {
        return SchuelerStatus.getMapID().get(status);
    }
    /**
     *
     * Ermittelt den Schülerstatus anhand der Bezeichnung.
     *
     * @param value	  die Bezeichnung des Schülerstatus
     *
     * @return der Schülerstatus oder null, falls die Bezeichnung ungültig ist
     *
     */
    static fromBezeichnung(value) {
        if (value === null)
            return null;
        return SchuelerStatus.getMapBezeichnungen().get(value.toUpperCase());
    }
    /**
     * Prüft, ob die übergebene ID für einen gültigen Schülerstatus
     * steht oder nicht
     *
     * @param id   die zu prüfende ID
     *
     * @return true, falls die ID gültig ist.
     */
    static isValidID(id) {
        if (id === null)
            return false;
        for (let status of SchuelerStatus.values())
            if (status.id === id)
                return true;
        return false;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof SchuelerStatus))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.SchuelerStatus'].includes(name);
    }
}
exports.SchuelerStatus = SchuelerStatus;
function cast_de_nrw_schule_svws_core_types_SchuelerStatus(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_SchuelerStatus = cast_de_nrw_schule_svws_core_types_SchuelerStatus;
//# sourceMappingURL=SchuelerStatus.js.map