import { JavaObject } from '../../../java/lang/JavaObject';
export declare class GostAbiturFach extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly LK1: GostAbiturFach;
    static readonly LK2: GostAbiturFach;
    static readonly AB3: GostAbiturFach;
    static readonly AB4: GostAbiturFach;
    readonly id: number;
    readonly kuerzel: String;
    readonly beschreibung: String;
    /**
     * Erstellt eine Abiturfachart für diese Aufzählung der Abiturfacharten.
     *
     * @param id             die ID bzw. Nummer der Abiturfachart
     * @param kuerzel        das Kürzel der Abiturfachart, welches auch in speziellen Kursarten genutzt wird
     * @param beschreibung   die textuelle Beschreibung der Abiturfachart
     */
    private constructor();
    /**
     *
     * Gibt die Abiturfachart anhand der ID zurück.
     *
     * @param id    die ID der Abiturfachart
     *
     * @return die Abiturfachart oder null falls die ID ungültig ist
     *
     */
    static fromID(id: Number | null): GostAbiturFach | null;
    /**
     *
     * Gibt die Abiturfachart anhand der ID (als String) zurück.
     *
     * @param strID    die ID der Abiturfachart (als String)
     *
     * @return die Abiturfachart oder null, falls die ID ungültig ist
     *
     */
    static fromIDString(strID: String | null): GostAbiturFach | null;
    /**
     *
     * Gibt die Abiturfachart anhand des Kürzels zurück.
     *
     * @param kuerzel    das Kürzel der Abiturfachart
     *
     * @return die Abiturfachart oder null, falls das Kürzel ungültig ist
     *
     */
    static fromKuerzel(kuerzel: String | null): GostAbiturFach | null;
    toString(): String;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: GostAbiturFach): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<GostAbiturFach>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): GostAbiturFach | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_gost_GostAbiturFach(obj: unknown): GostAbiturFach;
