import { JavaObject } from '../../../java/lang/JavaObject';
import { GostFach } from '../../../core/data/gost/GostFach';
import { List } from '../../../java/util/List';
export declare class GostFachbereich extends JavaObject {
    private static all;
    private static kuerzel_DEUTSCH;
    private static kuerzel_FREMDSPRACHE;
    private static kuerzel_KUNST_MUSIK;
    private static kuerzel_LITERARISCH_KUENSTLERISCH_ERSATZ;
    private static kuerzel_GESCHICHTE;
    private static kuerzel_SOZIALWISSENSCHAFTEN;
    private static kuerzel_PHILOSOPHIE;
    private static kuerzel_GESELLSCHAFTSWISSENSCHAFTLICH_SONSTIGE;
    private static kuerzel_MATHEMATIK;
    private static kuerzel_NATURWISSENSCHAFTLICH_KLASSISCH;
    private static kuerzel_NATURWISSENSCHAFTLICH_NEU_EINSETZEND;
    private static kuerzel_RELIGION;
    private static kuerzel_SPORT;
    static readonly SPRACHLICH_LITERARISCH_KUENSTLERISCH: GostFachbereich;
    static readonly DEUTSCH: GostFachbereich;
    static readonly FREMDSPRACHE: GostFachbereich;
    static readonly KUNST_MUSIK: GostFachbereich;
    static readonly LITERARISCH_KUENSTLERISCH: GostFachbereich;
    static readonly LITERARISCH_KUENSTLERISCH_ERSATZ: GostFachbereich;
    static readonly GESELLSCHAFTSWISSENSCHAFTLICH: GostFachbereich;
    static readonly GESCHICHTE: GostFachbereich;
    static readonly SOZIALWISSENSCHAFTEN: GostFachbereich;
    static readonly PHILOSOPHIE: GostFachbereich;
    static readonly GESELLSCHAFTSWISSENSCHAFTLICH_SONSTIGE: GostFachbereich;
    static readonly GESELLSCHAFTSWISSENSCHAFTLICH_MIT_RELIGION: GostFachbereich;
    static readonly MATHEMATIK: GostFachbereich;
    static readonly MATHEMATISCH_NATURWISSENSCHAFTLICH: GostFachbereich;
    static readonly NATURWISSENSCHAFTLICH: GostFachbereich;
    static readonly NATURWISSENSCHAFTLICH_KLASSISCH: GostFachbereich;
    static readonly NATURWISSENSCHAFTLICH_NEU_EINSETZEND: GostFachbereich;
    static readonly RELIGION: GostFachbereich;
    static readonly SPORT: GostFachbereich;
    private readonly kuerzel;
    /**
     * Erstellt einen neuen Fachbereich mit den übergebenen Kürzeln von Fächern
     *
     * @param kuerzel   die Kürzel der Fächer des Fachbereichs
     */
    private constructor();
    /**
     * Prüft, ob das übergebene Fach zu diesem Fachbereich gehört.
     *
     * @param fach   das zu prüfende Fach
     *
     * @return true, falls das Fach zu dem Fachbereich gehört, sonst false
     */
    hat(fach: GostFach | null): boolean;
    /**
     * Prüft, ob das Fach mit dem übergebenen Kürzel zu diesem Fachbereich gehört.
     *
     * @param kuerzel   das Kürzel des zu prüfenden Faches
     *
     * @return true, falls das Fach zu dem Fachbereich gehört, sonst false
     */
    hat(kuerzel: String | null): boolean;
    /**
     * Liefert alle Kürzel der Fächer zurück, die zu diesem Fachbereich gehören.
     *
     * @return alle Kürzel der Fächer dieses Fachbereichs
     */
    getAlleKuerzel(): Array<String>;
    /**
     * Ermittelt die, dem Fach zugehörigen, Fachbereiche anhand des Statistik-Kürzels
     *
     * @param kuerzel   das Statistik-Kürzel des Faches
     *
     * @return die zugehörigen Fachbereiche
     */
    static getBereiche(kuerzel: String | null): List<GostFachbereich>;
    /**
     * Ermittelt die, dem Fach zugehörigen, Fachbereiche
     *
     * @param fach   das Fach
     *
     * @return die zugehörigen Fachbereiche
     */
    static getBereiche(fach: GostFach | null): List<GostFachbereich>;
    /**
     * Liefert eine Liste mit allen Fachbereichen zurück.
     *
     * @return eine Liste mit allen Fachbereichen.
     */
    static values(): List<GostFachbereich>;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_gost_GostFachbereich(obj: unknown): GostFachbereich;
