import { JavaObject } from '../../../java/lang/JavaObject';
import { KursblockungInputRegel } from '../../../core/data/kursblockung/KursblockungInputRegel';
import { GostBlockungRegel } from '../../../core/data/gost/GostBlockungRegel';
export declare class GostKursblockungRegelTyp extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly UNDEFINIERT: GostKursblockungRegelTyp;
    static readonly SPERRE_SCHIENEN_VON_BIS: GostKursblockungRegelTyp;
    static readonly FIXIERE_KURS: GostKursblockungRegelTyp;
    static readonly SPERRE_SCHIENE_FUER_KURS: GostKursblockungRegelTyp;
    readonly id: number;
    private readonly paramCount;
    /**
     * Erstellt einen neuen Regel-Typ mit der angegeben ID.
     *
     * @param id   die ID des Regel-Typs
     */
    private constructor();
    /**
     * Ermittelt den Regel-Typ anhand seiner ID und gibt diesen zurück.
     *
     * @param id   die ID des Regel-Typs
     *
     * @return der Regel-Typ
     */
    static fromID(id: Number | null): GostKursblockungRegelTyp;
    /**
     * Gibt die Anzahl der Parameter für diesen Regel-Type zurück.
     *
     * @return die Anzahl der Parameter für diesen Regel-Type zurück.
     */
    getParamCount(): number;
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * Parameter.
     *
     * @param params   die Parameter für das Regel-Objekt
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn die Anzahl der Parameter nicht zu diesem Regel-Typ passt
     */
    create(...params: Array<Number>): KursblockungInputRegel;
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * ID und der angegebenen Parameter.
     *
     * @param id       die ID des Regel-Typs
     * @param params   die Parameter der zu erstellenden Regel
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn es keine gültige Regel für angegeben ID gibt oder die Anzahl
     *                                    der Parameter nicht zu dem Regel-Typ passt
     */
    static createByID(id: Number | null, ...params: Array<Number>): KursblockungInputRegel;
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * Regel.
     *
     * @param regel       die Regel als Core-DTO
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn es keine gültige Regel für angegeben ID gibt oder die Anzahl
     *                                    der Parameter nicht zu dem Regel-Typ passt
     */
    static createFrom(regel: GostBlockungRegel): KursblockungInputRegel;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: GostKursblockungRegelTyp): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<GostKursblockungRegelTyp>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): GostKursblockungRegelTyp | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_kursblockung_GostKursblockungRegelTyp(obj: unknown): GostKursblockungRegelTyp;
