"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_kursblockung_GostKursblockungRegelTyp = exports.GostKursblockungRegelTyp = void 0;
const JavaObject_1 = require("../../../java/lang/JavaObject");
const NullPointerException_1 = require("../../../java/lang/NullPointerException");
const KursblockungInputRegel_1 = require("../../../core/data/kursblockung/KursblockungInputRegel");
const IllegalArgumentException_1 = require("../../../java/lang/IllegalArgumentException");
class GostKursblockungRegelTyp extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static UNDEFINIERT = new GostKursblockungRegelTyp("UNDEFINIERT", 0, 0);
    static SPERRE_SCHIENEN_VON_BIS = new GostKursblockungRegelTyp("SPERRE_SCHIENEN_VON_BIS", 1, 1);
    static FIXIERE_KURS = new GostKursblockungRegelTyp("FIXIERE_KURS", 2, 2);
    static SPERRE_SCHIENE_FUER_KURS = new GostKursblockungRegelTyp("SPERRE_SCHIENE_FUER_KURS", 3, 3);
    id;
    paramCount;
    /**
     * Erstellt einen neuen Regel-Typ mit der angegeben ID.
     *
     * @param id   die ID des Regel-Typs
     */
    constructor(name, ordinal, id) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        GostKursblockungRegelTyp.all_values_by_ordinal.push(this);
        GostKursblockungRegelTyp.all_values_by_name.set(name, this);
        this.id = id;
        switch (id) {
            case 0:
                this.paramCount = 0;
                break;
            case 1:
                this.paramCount = 3;
                break;
            case 2:
            case 3:
                this.paramCount = 2;
                break;
            default:
                throw new IllegalArgumentException_1.IllegalArgumentException();
        }
    }
    /**
     * Ermittelt den Regel-Typ anhand seiner ID und gibt diesen zurück.
     *
     * @param id   die ID des Regel-Typs
     *
     * @return der Regel-Typ
     */
    static fromID(id) {
        if ((id === null) || (id < 0))
            return GostKursblockungRegelTyp.UNDEFINIERT;
        let values = GostKursblockungRegelTyp.values();
        if (id > values.length)
            return GostKursblockungRegelTyp.UNDEFINIERT;
        return values[id.valueOf()];
    }
    /**
     * Gibt die Anzahl der Parameter für diesen Regel-Type zurück.
     *
     * @return die Anzahl der Parameter für diesen Regel-Type zurück.
     */
    getParamCount() {
        return this.paramCount;
    }
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * Parameter.
     *
     * @param params   die Parameter für das Regel-Objekt
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn die Anzahl der Parameter nicht zu diesem Regel-Typ passt
     */
    create(...params) {
        if ((params === null) || (params.length !== this.paramCount))
            throw new IllegalArgumentException_1.IllegalArgumentException();
        let regel = new KursblockungInputRegel_1.KursblockungInputRegel();
        regel.id = this.id;
        regel.daten = Array(params.length).fill(null);
        for (let i = 0; i < params.length; i++) {
            let data = params[i];
            if (data === null)
                throw new NullPointerException_1.NullPointerException();
            regel.daten[i] = data;
        }
        return regel;
    }
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * ID und der angegebenen Parameter.
     *
     * @param id       die ID des Regel-Typs
     * @param params   die Parameter der zu erstellenden Regel
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn es keine gültige Regel für angegeben ID gibt oder die Anzahl
     *                                    der Parameter nicht zu dem Regel-Typ passt
     */
    static createByID(id, ...params) {
        let typ = GostKursblockungRegelTyp.fromID(id);
        if (JavaObject_1.JavaObject.equalsTranspiler(typ, (GostKursblockungRegelTyp.UNDEFINIERT)))
            throw new IllegalArgumentException_1.IllegalArgumentException("Ungültiger Regel-Typ");
        return typ.create(...params);
    }
    /**
     * Erstellt ein neues Regel-Objekt der Klasse {@link KursblockungInputRegel} mithilfe der angegebenen
     * Regel.
     *
     * @param regel       die Regel als Core-DTO
     *
     * @return das Regel-Objekt
     *
     * @throws IllegalArgumentException   wenn es keine gültige Regel für angegeben ID gibt oder die Anzahl
     *                                    der Parameter nicht zu dem Regel-Typ passt
     */
    static createFrom(regel) {
        return GostKursblockungRegelTyp.createByID(regel.typ, ...regel.parameter.toArray(Array(0).fill(null)));
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof GostKursblockungRegelTyp))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.kursblockung.GostKursblockungRegelTyp'].includes(name);
    }
}
exports.GostKursblockungRegelTyp = GostKursblockungRegelTyp;
function cast_de_nrw_schule_svws_core_types_kursblockung_GostKursblockungRegelTyp(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_kursblockung_GostKursblockungRegelTyp = cast_de_nrw_schule_svws_core_types_kursblockung_GostKursblockungRegelTyp;
//# sourceMappingURL=GostKursblockungRegelTyp.js.map