import { JavaObject } from '../../../java/lang/JavaObject';
import { Schulform } from '../../../core/types/statkue/Schulform';
import { List } from '../../../java/util/List';
export declare class Foerderschwerpunkt extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly KEINER: Foerderschwerpunkt;
    static readonly BL: Foerderschwerpunkt;
    static readonly EZ: Foerderschwerpunkt;
    static readonly GB: Foerderschwerpunkt;
    static readonly GH: Foerderschwerpunkt;
    static readonly KB: Foerderschwerpunkt;
    static readonly KR: Foerderschwerpunkt;
    static readonly LB: Foerderschwerpunkt;
    static readonly PE: Foerderschwerpunkt;
    static readonly PF: Foerderschwerpunkt;
    static readonly PL: Foerderschwerpunkt;
    static readonly PS: Foerderschwerpunkt;
    static readonly SB: Foerderschwerpunkt;
    static readonly SG: Foerderschwerpunkt;
    static readonly SH: Foerderschwerpunkt;
    static readonly XX: Foerderschwerpunkt;
    readonly kuerzel: String;
    readonly beschreibung: String;
    readonly schulformenKuerzel: List<String>;
    private schulformen;
    readonly gueltigVon: Number | null;
    readonly gueltigBis: Number | null;
    /**
     * Erzeugt einen neuen Förderschwerpunkt in der Aufzählung.
     *
     * @param kuerzel              das Kürzel, welche im Rahmen der amtlichen Schulstatistik verwendet wird
     * @param beschreibung         die textuelle Bezeichnung des Förderschwerpunktes
     * @param schulformenKuerzel   eine Liste mit den Kürzeln der Schulformen, bei denen der Förderschwerpunkt vorkommt
     */
    private constructor();
    /**
     * Liefert alle zulässigen Förderschwerpunkte für die angegeben Schulform.
     *
     * @param schulform   die Schulform
     *
     * @return die bei der Schulform zulässigen Förderschwerpunkte
     */
    static get(schulform: Schulform | null): List<Foerderschwerpunkt>;
    /**
     * Liefert alle Schulformen zurück, bei welchen der Förderschwerpunkt vorkommt.
     *
     * @return eine Liste der Schulformen
     */
    getSchulformen(): List<Schulform>;
    /**
     * Liefert den Förderschwerpunkt anhand des übergebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel des gesuchten Förderschwerpunktes
     *
     * @return der gesuchte Förderschwerpunkt oder null, wenn kein solcher existiert
     */
    static getByKuerzel(kuerzel: String | null): Foerderschwerpunkt | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: Foerderschwerpunkt): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<Foerderschwerpunkt>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): Foerderschwerpunkt | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_statkue_Foerderschwerpunkt(obj: unknown): Foerderschwerpunkt;
