import { JavaObject } from '../../../java/lang/JavaObject';
import { LehrerKatalogAbgangsgrundEintrag } from '../../../core/data/lehrer/LehrerKatalogAbgangsgrundEintrag';
export declare class LehrerAbgangsgrund extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly RUHEST: LehrerAbgangsgrund;
    static readonly UNFAEHIGK: LehrerAbgangsgrund;
    static readonly TOD: LehrerAbgangsgrund;
    static readonly AndBuLand: LehrerAbgangsgrund;
    static readonly WECHSEL: LehrerAbgangsgrund;
    static readonly BEFRIST: LehrerAbgangsgrund;
    static readonly SONSTIG: LehrerAbgangsgrund;
    static VERSION: number;
    readonly daten: LehrerKatalogAbgangsgrundEintrag;
    readonly historie: Array<LehrerKatalogAbgangsgrundEintrag>;
    /**
     * Erzeugt einen neuen Grund in der Aufzählung.
     *
     * @param historie   die Historie des Abgangsgrundes, welches ein Array von {@link LehrerKatalogAbgangsgrundEintrag} ist
     */
    private constructor();
    /**
     * Gibt den Grund anhand der angegebenen ID zurück.
     *
     * @param id   die ID des Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls die ID ungültig ist
     */
    static getByID(id: number): LehrerAbgangsgrund | null;
    /**
     * Gibt den Grund anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel des Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls das kuerzel ungültig ist
     */
    static getByKuerzel(kuerzel: String | null): LehrerAbgangsgrund | null;
    /**
     * Gibt den Grund anhand des angegebenen Schlüssels der
     * amtlichen Schulstatistik zurück.
     *
     * @param schluessel   der Schlüssel der amtlichen Schulstatistik
     *                     für den Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls der Schlüssel ungültig ist
     */
    static getByASDSchluessel(schluessel: String | null): LehrerAbgangsgrund | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: LehrerAbgangsgrund): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<LehrerAbgangsgrund>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): LehrerAbgangsgrund | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_statkue_LehrerAbgangsgrund(obj: unknown): LehrerAbgangsgrund;
