"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerAbgangsgrund = exports.LehrerAbgangsgrund = void 0;
const JavaObject_1 = require("../../../java/lang/JavaObject");
const LehrerKatalogAbgangsgrundEintrag_1 = require("../../../core/data/lehrer/LehrerKatalogAbgangsgrundEintrag");
class LehrerAbgangsgrund extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static RUHEST = new LehrerAbgangsgrund("RUHEST", 0, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(1, "RUHEST", "Eintritt in den Ruhestand", "11", null, null)]);
    static UNFAEHIGK = new LehrerAbgangsgrund("UNFAEHIGK", 1, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(2, "UNFÄHIGK", "Dienst-, Erwerbs-, Berufsunfähigkeit", "12", null, null)]);
    static TOD = new LehrerAbgangsgrund("TOD", 2, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(3, "TOD", "Tod", "13", null, null)]);
    static AndBuLand = new LehrerAbgangsgrund("AndBuLand", 3, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(4, "AndBuLand", "Übertritt in den Schuldienst eines anderen Bundeslandes", "14", null, null)]);
    static WECHSEL = new LehrerAbgangsgrund("WECHSEL", 4, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(5, "WECHSEL", "Wechsel innerhalb des Landes von der berichtenden Schule an eine andere Schule", "15", null, null)]);
    static BEFRIST = new LehrerAbgangsgrund("BEFRIST", 5, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(6, "BEFRIST", "Befristete Abgänge", "16", null, null)]);
    static SONSTIG = new LehrerAbgangsgrund("SONSTIG", 6, [new LehrerKatalogAbgangsgrundEintrag_1.LehrerKatalogAbgangsgrundEintrag(7, "SONSTIG", "Sonstige Abgänge", "17", null, null)]);
    static VERSION = 1;
    daten;
    historie;
    /**
     * Erzeugt einen neuen Grund in der Aufzählung.
     *
     * @param historie   die Historie des Abgangsgrundes, welches ein Array von {@link LehrerKatalogAbgangsgrundEintrag} ist
     */
    constructor(name, ordinal, historie) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        LehrerAbgangsgrund.all_values_by_ordinal.push(this);
        LehrerAbgangsgrund.all_values_by_name.set(name, this);
        this.historie = historie;
        this.daten = historie[historie.length - 1];
    }
    /**
     * Gibt den Grund anhand der angegebenen ID zurück.
     *
     * @param id   die ID des Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls die ID ungültig ist
     */
    static getByID(id) {
        for (let grund of LehrerAbgangsgrund.values())
            if (grund.daten.id === id)
                return grund;
        return null;
    }
    /**
     * Gibt den Grund anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel des Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls das kuerzel ungültig ist
     */
    static getByKuerzel(kuerzel) {
        for (let grund of LehrerAbgangsgrund.values())
            if (JavaObject_1.JavaObject.equalsTranspiler(grund.daten.kuerzel, (kuerzel)))
                return grund;
        return null;
    }
    /**
     * Gibt den Grund anhand des angegebenen Schlüssels der
     * amtlichen Schulstatistik zurück.
     *
     * @param schluessel   der Schlüssel der amtlichen Schulstatistik
     *                     für den Grundes
     *
     * @return der Grund für das Verlassen der Schule durch den Lehrer oder null, falls der Schlüssel ungültig ist
     */
    static getByASDSchluessel(schluessel) {
        for (let grund of LehrerAbgangsgrund.values())
            if (JavaObject_1.JavaObject.equalsTranspiler(grund.daten.schluessel, (schluessel)))
                return grund;
        return null;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof LehrerAbgangsgrund))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.statkue.LehrerAbgangsgrund'].includes(name);
    }
}
exports.LehrerAbgangsgrund = LehrerAbgangsgrund;
function cast_de_nrw_schule_svws_core_types_statkue_LehrerAbgangsgrund(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerAbgangsgrund = cast_de_nrw_schule_svws_core_types_statkue_LehrerAbgangsgrund;
//# sourceMappingURL=LehrerAbgangsgrund.js.map