import { JavaObject } from '../../../java/lang/JavaObject';
import { LehrerKatalogEinsatzstatusEintrag } from '../../../core/data/lehrer/LehrerKatalogEinsatzstatusEintrag';
export declare class LehrerEinsatzstatus extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly A: LehrerEinsatzstatus;
    static readonly B: LehrerEinsatzstatus;
    static readonly DEFAULT: LehrerEinsatzstatus;
    static VERSION: number;
    readonly daten: LehrerKatalogEinsatzstatusEintrag;
    readonly historie: Array<LehrerKatalogEinsatzstatusEintrag>;
    private static readonly _statusByID;
    private static readonly _statusByKuerzel;
    /**
     * Erzeugt einen neuen Einsatzstatus in der Aufzählung.
     *
     * @param historie   die Historie des Einsatzstatus, welches ein Array von {@link LehrerKatalogEinsatzstatusEintrag} ist
     */
    private constructor();
    /**
     * Gibt eine Map von der ID des Einsatzstatus auf den zugehörigen Einsatzstatus
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von der ID des Einsatzstatus auf den zugehörigen Einsatzstatus
     */
    private static getMapStatusByID;
    /**
     * Gibt eine Map von de Kürzel des Einsatzstatus auf den zugehörigen Einsatzstatus
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von de Kürzel des Einsatzstatus auf den zugehörigen Einsatzstatus
     */
    private static getMapStatusByKuerzel;
    /**
     * Gibt den Einsatzstatus anhand der angegebenen ID zurück.
     *
     * @param id   die ID des Einsatzstatus
     *
     * @return der Einsatzstatus oder null, falls die ID ungültig ist
     */
    static getByID(id: number): LehrerEinsatzstatus | null;
    /**
     * Gibt den Einsatzstatus anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel des Einsatzstatus
     *
     * @return der Einsatzstatus oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel: String | null): LehrerEinsatzstatus | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: LehrerEinsatzstatus): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<LehrerEinsatzstatus>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): LehrerEinsatzstatus | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_statkue_LehrerEinsatzstatus(obj: unknown): LehrerEinsatzstatus;
