import { JavaObject } from '../../../java/lang/JavaObject';
import { LehrerKatalogFachrichtungAnerkennungEintrag } from '../../../core/data/lehrer/LehrerKatalogFachrichtungAnerkennungEintrag';
export declare class LehrerFachrichtungAnerkennung extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly ID4: LehrerFachrichtungAnerkennung;
    static readonly ID5: LehrerFachrichtungAnerkennung;
    static readonly ID6: LehrerFachrichtungAnerkennung;
    static readonly ID7: LehrerFachrichtungAnerkennung;
    static VERSION: number;
    readonly daten: LehrerKatalogFachrichtungAnerkennungEintrag;
    readonly historie: Array<LehrerKatalogFachrichtungAnerkennungEintrag>;
    private static readonly _anerkennungenByID;
    private static readonly _anerkennungenByKuerzel;
    /**
     * Erzeugt neue Anerkennung für Fachrichtungen in der Aufzählung.
     *
     * @param historie   die Historie der Anerkennung für Fachrichtungen, welches ein Array von {@link LehrerKatalogFachrichtungAnerkennungEintrag} ist
     */
    private constructor();
    /**
     * Gibt eine Map von den IDs der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     */
    private static getMapAnerkennungenByID;
    /**
     * Gibt eine Map von den Kürzeln der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Kürzeln der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     */
    private static getMapAnerkennungenByKuerzel;
    /**
     * Gibt die Fachrichtungsanerkennung anhand der angegebenen ID zurück.
     *
     * @param id   die ID der Fachrichtungsanerkennung
     *
     * @return die Fachrichtungsanerkennung oder null, falls die ID ungültig ist
     */
    static getByID(id: number): LehrerFachrichtungAnerkennung | null;
    /**
     * Gibt die Fachrichtungsanerkennung anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel der Fachrichtungsanerkennung
     *
     * @return die Fachrichtungsanerkennung oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel: String | null): LehrerFachrichtungAnerkennung | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: LehrerFachrichtungAnerkennung): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<LehrerFachrichtungAnerkennung>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): LehrerFachrichtungAnerkennung | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_statkue_LehrerFachrichtungAnerkennung(obj: unknown): LehrerFachrichtungAnerkennung;
