"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerFachrichtungAnerkennung = exports.LehrerFachrichtungAnerkennung = void 0;
const JavaObject_1 = require("../../../java/lang/JavaObject");
const HashMap_1 = require("../../../java/util/HashMap");
const LehrerKatalogFachrichtungAnerkennungEintrag_1 = require("../../../core/data/lehrer/LehrerKatalogFachrichtungAnerkennungEintrag");
class LehrerFachrichtungAnerkennung extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static ID4 = new LehrerFachrichtungAnerkennung("ID4", 0, [new LehrerKatalogFachrichtungAnerkennungEintrag_1.LehrerKatalogFachrichtungAnerkennungEintrag(4, "1", "erworben durch LABG/OVP bzw. Laufbahnverordnung", null, null)]);
    static ID5 = new LehrerFachrichtungAnerkennung("ID5", 1, [new LehrerKatalogFachrichtungAnerkennungEintrag_1.LehrerKatalogFachrichtungAnerkennungEintrag(5, "2", "Unterrichtserlaubnis (z. B. Zertifikatskurs)", null, null)]);
    static ID6 = new LehrerFachrichtungAnerkennung("ID6", 2, [new LehrerKatalogFachrichtungAnerkennungEintrag_1.LehrerKatalogFachrichtungAnerkennungEintrag(6, "3", "mehrjähriger Unterricht ohne Lehramtsprüfung oder Unterrichtserlaubnis", null, null)]);
    static ID7 = new LehrerFachrichtungAnerkennung("ID7", 3, [new LehrerKatalogFachrichtungAnerkennungEintrag_1.LehrerKatalogFachrichtungAnerkennungEintrag(7, "9", "sonstige", null, null)]);
    static VERSION = 1;
    daten;
    historie;
    static _anerkennungenByID = new HashMap_1.HashMap();
    static _anerkennungenByKuerzel = new HashMap_1.HashMap();
    /**
     * Erzeugt neue Anerkennung für Fachrichtungen in der Aufzählung.
     *
     * @param historie   die Historie der Anerkennung für Fachrichtungen, welches ein Array von {@link LehrerKatalogFachrichtungAnerkennungEintrag} ist
     */
    constructor(name, ordinal, historie) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        LehrerFachrichtungAnerkennung.all_values_by_ordinal.push(this);
        LehrerFachrichtungAnerkennung.all_values_by_name.set(name, this);
        this.historie = historie;
        this.daten = historie[historie.length - 1];
    }
    /**
     * Gibt eine Map von den IDs der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     */
    static getMapAnerkennungenByID() {
        if (LehrerFachrichtungAnerkennung._anerkennungenByID.size() === 0)
            for (let l of LehrerFachrichtungAnerkennung.values())
                LehrerFachrichtungAnerkennung._anerkennungenByID.put(l.daten.id, l);
        return LehrerFachrichtungAnerkennung._anerkennungenByID;
    }
    /**
     * Gibt eine Map von den Kürzeln der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Kürzeln der Fachrichtungsanerkennungen auf die zugehörigen Fachrichtungsanerkennungen
     */
    static getMapAnerkennungenByKuerzel() {
        if (LehrerFachrichtungAnerkennung._anerkennungenByKuerzel.size() === 0)
            for (let l of LehrerFachrichtungAnerkennung.values())
                LehrerFachrichtungAnerkennung._anerkennungenByKuerzel.put(l.daten.kuerzel, l);
        return LehrerFachrichtungAnerkennung._anerkennungenByKuerzel;
    }
    /**
     * Gibt die Fachrichtungsanerkennung anhand der angegebenen ID zurück.
     *
     * @param id   die ID der Fachrichtungsanerkennung
     *
     * @return die Fachrichtungsanerkennung oder null, falls die ID ungültig ist
     */
    static getByID(id) {
        return LehrerFachrichtungAnerkennung.getMapAnerkennungenByID().get(id);
    }
    /**
     * Gibt die Fachrichtungsanerkennung anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel der Fachrichtungsanerkennung
     *
     * @return die Fachrichtungsanerkennung oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel) {
        return LehrerFachrichtungAnerkennung.getMapAnerkennungenByKuerzel().get(kuerzel);
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof LehrerFachrichtungAnerkennung))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.statkue.LehrerFachrichtungAnerkennung'].includes(name);
    }
}
exports.LehrerFachrichtungAnerkennung = LehrerFachrichtungAnerkennung;
function cast_de_nrw_schule_svws_core_types_statkue_LehrerFachrichtungAnerkennung(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerFachrichtungAnerkennung = cast_de_nrw_schule_svws_core_types_statkue_LehrerFachrichtungAnerkennung;
//# sourceMappingURL=LehrerFachrichtungAnerkennung.js.map