import { JavaObject } from '../../../java/lang/JavaObject';
import { LehrerKatalogMehrleistungsartEintrag } from '../../../core/data/lehrer/LehrerKatalogMehrleistungsartEintrag';
export declare class LehrerMehrleistungArt extends JavaObject {
    /** the name of the enumeration value */
    private readonly __name;
    /** the ordinal value for the enumeration value */
    private readonly __ordinal;
    /** an array containing all values of this enumeration */
    private static readonly all_values_by_ordinal;
    /** an array containing all values of this enumeration indexed by their name*/
    private static readonly all_values_by_name;
    static readonly ID_100: LehrerMehrleistungArt;
    static readonly ID_110: LehrerMehrleistungArt;
    static readonly ID_150: LehrerMehrleistungArt;
    static readonly ID_160: LehrerMehrleistungArt;
    static readonly ID_165: LehrerMehrleistungArt;
    static readonly ID_170: LehrerMehrleistungArt;
    static VERSION: number;
    readonly daten: LehrerKatalogMehrleistungsartEintrag;
    readonly historie: Array<LehrerKatalogMehrleistungsartEintrag>;
    private static readonly _artenByID;
    private static readonly _artenByKuerzel;
    /**
     * Erzeugt eine neue Art von Mehrleistung in der Aufzählung.
     *
     * @param historie   die Historie der Art von Mehrleistung, welches ein Array von {@link LehrerKatalogMehrleistungsartEintrag} ist
     */
    private constructor();
    /**
     * Gibt eine Map von den IDs der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     */
    private static getMapArtenByID;
    /**
     * Gibt eine Map von den Kürzeln der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Kürzeln der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     */
    private static getMapArtenByKuerzel;
    /**
     * Gibt die Art der Mehrleistung anhand der angegebenen ID zurück.
     *
     * @param id   die ID der Art der Mehrleistung
     *
     * @return die Art der Mehrleistung oder null, falls die ID ungültig ist
     */
    static getByID(id: number): LehrerMehrleistungArt | null;
    /**
     * Gibt die Art der Mehrleistung anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel der Art der Mehrleistung
     *
     * @return die Art der Mehrleistung oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel: String | null): LehrerMehrleistungArt | null;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    private name;
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    private ordinal;
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString(): String;
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other: JavaObject): boolean;
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode(): number;
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other: LehrerMehrleistungArt): number;
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values(): Array<LehrerMehrleistungArt>;
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name: String): LehrerMehrleistungArt | null;
    isTranspiledInstanceOf(name: string): boolean;
}
export declare function cast_de_nrw_schule_svws_core_types_statkue_LehrerMehrleistungArt(obj: unknown): LehrerMehrleistungArt;
