"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerMehrleistungArt = exports.LehrerMehrleistungArt = void 0;
const JavaObject_1 = require("../../../java/lang/JavaObject");
const HashMap_1 = require("../../../java/util/HashMap");
const LehrerKatalogMehrleistungsartEintrag_1 = require("../../../core/data/lehrer/LehrerKatalogMehrleistungsartEintrag");
class LehrerMehrleistungArt extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static ID_100 = new LehrerMehrleistungArt("ID_100", 0, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(1, "100", "Beschäftigungsphase Sabbatjahr", null, null)]);
    static ID_110 = new LehrerMehrleistungArt("ID_110", 1, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(2, "110", "Mehrarbeit (angeordnet und regelmäßig)", null, null)]);
    static ID_150 = new LehrerMehrleistungArt("ID_150", 2, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(3, "150", "Aufrundung der Pflichtstundenzahl wegen Abrundung im folgenden Schuljahr ", null, null)]);
    static ID_160 = new LehrerMehrleistungArt("ID_160", 3, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(4, "160", "Überschreitung der Pflichtstundenzahl aus organisatorischen Gründen (z. B. Epochenunterricht)", null, null)]);
    static ID_165 = new LehrerMehrleistungArt("ID_165", 4, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(6, "165", "Überschreitung der Pflichtstundenzahl wegen COVID-19", null, null)]);
    static ID_170 = new LehrerMehrleistungArt("ID_170", 5, [new LehrerKatalogMehrleistungsartEintrag_1.LehrerKatalogMehrleistungsartEintrag(5, "170", "Überschreitung der Pflichtstundenzahl wegen Pflichstunden-Bandbreite", null, null)]);
    static VERSION = 1;
    daten;
    historie;
    static _artenByID = new HashMap_1.HashMap();
    static _artenByKuerzel = new HashMap_1.HashMap();
    /**
     * Erzeugt eine neue Art von Mehrleistung in der Aufzählung.
     *
     * @param historie   die Historie der Art von Mehrleistung, welches ein Array von {@link LehrerKatalogMehrleistungsartEintrag} ist
     */
    constructor(name, ordinal, historie) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        LehrerMehrleistungArt.all_values_by_ordinal.push(this);
        LehrerMehrleistungArt.all_values_by_name.set(name, this);
        this.historie = historie;
        this.daten = historie[historie.length - 1];
    }
    /**
     * Gibt eine Map von den IDs der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den IDs der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     */
    static getMapArtenByID() {
        if (LehrerMehrleistungArt._artenByID.size() === 0)
            for (let g of LehrerMehrleistungArt.values())
                LehrerMehrleistungArt._artenByID.put(g.daten.id, g);
        return LehrerMehrleistungArt._artenByID;
    }
    /**
     * Gibt eine Map von den Kürzeln der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     * zurück. Sollte diese noch nicht initialisiert sein, so wird sie initielisiert.
     *
     * @return die Map von den Kürzeln der Mehrleistungsarten auf die zugehörigen Mehrleistungsarten
     */
    static getMapArtenByKuerzel() {
        if (LehrerMehrleistungArt._artenByKuerzel.size() === 0)
            for (let g of LehrerMehrleistungArt.values())
                LehrerMehrleistungArt._artenByKuerzel.put(g.daten.kuerzel, g);
        return LehrerMehrleistungArt._artenByKuerzel;
    }
    /**
     * Gibt die Art der Mehrleistung anhand der angegebenen ID zurück.
     *
     * @param id   die ID der Art der Mehrleistung
     *
     * @return die Art der Mehrleistung oder null, falls die ID ungültig ist
     */
    static getByID(id) {
        return LehrerMehrleistungArt.getMapArtenByID().get(id);
    }
    /**
     * Gibt die Art der Mehrleistung anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel der Art der Mehrleistung
     *
     * @return die Art der Mehrleistung oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel) {
        return LehrerMehrleistungArt.getMapArtenByKuerzel().get(kuerzel);
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof LehrerMehrleistungArt))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.statkue.LehrerMehrleistungArt'].includes(name);
    }
}
exports.LehrerMehrleistungArt = LehrerMehrleistungArt;
function cast_de_nrw_schule_svws_core_types_statkue_LehrerMehrleistungArt(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerMehrleistungArt = cast_de_nrw_schule_svws_core_types_statkue_LehrerMehrleistungArt;
//# sourceMappingURL=LehrerMehrleistungArt.js.map