"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerZugangsgrund = exports.LehrerZugangsgrund = void 0;
const JavaObject_1 = require("../../../java/lang/JavaObject");
const LehrerKatalogZugangsgrundEintrag_1 = require("../../../core/data/lehrer/LehrerKatalogZugangsgrundEintrag");
class LehrerZugangsgrund extends JavaObject_1.JavaObject {
    /** the name of the enumeration value */
    __name;
    /** the ordinal value for the enumeration value */
    __ordinal;
    /** an array containing all values of this enumeration */
    static all_values_by_ordinal = [];
    /** an array containing all values of this enumeration indexed by their name*/
    static all_values_by_name = new Map();
    static NEU = new LehrerZugangsgrund("NEU", 0, [new LehrerKatalogZugangsgrundEintrag_1.LehrerKatalogZugangsgrundEintrag(1, "NEU", "Neueintritt in den Schuldienst mit abgelegter 2. Staatsprüfung oder anderweitig erfüllter Eingangsvoraussetzung", "1", null, null)]);
    static AndBuLand = new LehrerZugangsgrund("AndBuLand", 1, [new LehrerKatalogZugangsgrundEintrag_1.LehrerKatalogZugangsgrundEintrag(2, "AndBuLand", "Übertritt aus dem Schuldienst eines anderen Bundeslandes", "2", null, null)]);
    static WECHSEL = new LehrerZugangsgrund("WECHSEL", 2, [new LehrerKatalogZugangsgrundEintrag_1.LehrerKatalogZugangsgrundEintrag(3, "WECHSEL", "Wechsel innerhalb des Landes von einer anderen Schule an die berichtende Schule", "3", null, null)]);
    static WIEDER = new LehrerZugangsgrund("WIEDER", 3, [new LehrerKatalogZugangsgrundEintrag_1.LehrerKatalogZugangsgrundEintrag(4, "WIEDER", "Wiedereintritt in den Schuldienst", "4", null, null)]);
    static SONSTIG = new LehrerZugangsgrund("SONSTIG", 4, [new LehrerKatalogZugangsgrundEintrag_1.LehrerKatalogZugangsgrundEintrag(5, "SONSTIG", "Sonstige Zugänge", "5", null, null)]);
    static VERSION = 1;
    daten;
    historie;
    /**
     * Erzeugt einen neuen Grund in der Aufzählung.
     *
     * @param historie   die Historie des Zugangsgrundes, welches ein Array von {@link LehrerKatalogZugangsgrundEintrag} ist
     */
    constructor(name, ordinal, historie) {
        super();
        this.__name = name;
        this.__ordinal = ordinal;
        LehrerZugangsgrund.all_values_by_ordinal.push(this);
        LehrerZugangsgrund.all_values_by_name.set(name, this);
        this.historie = historie;
        this.daten = historie[historie.length - 1];
    }
    /**
     * Gibt den Grund anhand der angegebenen ID zurück.
     *
     * @param id   die ID des Grundes
     *
     * @return der Grund für das Kommen des Lehrers an die Schule oder null, falls die ID ungültig ist
     */
    static getByID(id) {
        for (let grund of LehrerZugangsgrund.values())
            if (grund.daten.id === id)
                return grund;
        return null;
    }
    /**
     * Gibt den Grund anhand des angegebenen Kürzels zurück.
     *
     * @param kuerzel   das Kürzel des Grundes
     *
     * @return der Grund für das Kommen des Lehrers an die Schule oder null, falls das Kürzel ungültig ist
     */
    static getByKuerzel(kuerzel) {
        for (let grund of LehrerZugangsgrund.values())
            if (JavaObject_1.JavaObject.equalsTranspiler(grund.daten.kuerzel, (kuerzel)))
                return grund;
        return null;
    }
    /**
     * Gibt den Grund anhand des angegebenen Schlüssels der
     * amtlichen Schulstatistik zurück.
     *
     * @param schluessel   der Schlüssel der amtlichen Schulstatistik
     *                     für den Grundes
     *
     * @return der Grund für das Kommen des Lehrers an die Schule oder null, falls der Schlüssel ungültig ist
     */
    static getByASDSchluessel(schluessel) {
        for (let grund of LehrerZugangsgrund.values())
            if (JavaObject_1.JavaObject.equalsTranspiler(grund.daten.schluessel, (schluessel)))
                return grund;
        return null;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    name() {
        return this.__name;
    }
    /**
     * Returns the ordinal value of this enumeration value.
     *
     * @returns the ordinal value
     */
    ordinal() {
        return this.__ordinal;
    }
    /**
     * Returns the name of this enumeration value.
     *
     * @returns the name
     */
    toString() {
        return this.__name;
    }
    /**
     * Returns true if this and the other enumeration values are equal.
     *
     * @param other   the other enumeration value
     *
     * @returns true if they are equal and false otherwise
     */
    equals(other) {
        if (!(other instanceof LehrerZugangsgrund))
            return false;
        return this === other;
    }
    /**
     * Returns the ordinal value as hashcode, since the ordinal value is unique.
     *
     * @returns the ordinal value as hashcode
     */
    hashCode() {
        return this.__ordinal;
    }
    /**
     * Compares this enumeration value with the other enumeration value by their ordinal value.
     *
     * @param other   the other enumeration value
     *
     * @returns a negative, zero or postive value as this enumeration value is less than, equal to
     *          or greater than the other enumeration value
     */
    compareTo(other) {
        return this.__ordinal - other.__ordinal;
    }
    /**
     * Returns an array with enumeration values.
     *
     * @returns the array with enumeration values
     */
    static values() {
        return [...this.all_values_by_ordinal];
    }
    /**
     * Returns the enumeration value with the specified name.
     *
     * @param name   the name of the enumeration value
     *
     * @returns the enumeration values or null
     */
    static valueOf(name) {
        let tmp = this.all_values_by_name.get(name);
        return (!tmp) ? null : tmp;
    }
    isTranspiledInstanceOf(name) {
        return ['de.nrw.schule.svws.core.types.statkue.LehrerZugangsgrund'].includes(name);
    }
}
exports.LehrerZugangsgrund = LehrerZugangsgrund;
function cast_de_nrw_schule_svws_core_types_statkue_LehrerZugangsgrund(obj) {
    return obj;
}
exports.cast_de_nrw_schule_svws_core_types_statkue_LehrerZugangsgrund = cast_de_nrw_schule_svws_core_types_statkue_LehrerZugangsgrund;
//# sourceMappingURL=LehrerZugangsgrund.js.map